""" Qaekwy """

__copyright__ = "©2023"
__license__ = "European Union Public Licence 1.2 (EUPL 1.2)"
__software__ = "Qaekwy"
__author__ = "Alexis LE GOADEC"
__author_email__ = "alex@qaekwy.io"
__version__ = "0.1.4"
